local db, duration
local format = string.format
local floor = math.floor
local GetTime = GetTime
local GetGameTime = GetGameTime


local addon = CreateFrame("Frame", "Broker_AFKTimerDropDown", nil, "UIDropDownMenuTemplate")
addon:RegisterEvent("ADDON_LOADED")
addon:RegisterEvent("PLAYER_LOGIN")
addon:RegisterEvent("PLAYER_LEAVING_WORLD")
addon:RegisterEvent("PLAYER_GUILD_UPDATE")
addon:RegisterEvent("CHAT_MSG_SYSTEM")
addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)


local function onUpdate(self)
	local counter = ""
	if db.showDuration then
		duration = floor(GetTime() - db.startUptime)
		counter = " ("..addon:SecondsToTime(duration)..")"
	end
	addon.dataobj.text = "AFK since "..db.startTimestamp..counter
end


-- dropdown menu items
local dropdown = {
	{
		text = "Show duration",
		func = function() db.showDuration = not db.showDuration end,
		checked = function() return db.showDuration end,
	},
	{
		text = "Use local time",
		func = function() db.localTime = not db.localTime end,
		checked = function() return db.localTime end,
	}
}


-- Broker data object
addon.dataobj = LibStub("LibDataBroker-1.1"):NewDataObject("Broker_AFKTimer", {
	type = "data source",
	text = "Not AFK",
	icon = "Interface\\Icons\\Spell_Nature_TimeStop",
	label = "AFK Timer",
	OnClick = function(self, button)
		if button == "LeftButton" then
			SendChatMessage("", "AFK")
		elseif button == "RightButton" then
			EasyMenu(dropdown, addon, self, 0, 0, "MENU")
		end
	end,
	OnTooltipShow = function()
		GameTooltip:AddLine("|cffffffffBroker_AFKTimer")
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(db.note and "You are AFK: |cffffffff"..db.note or "You are not AFK.")
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine("|cff00ff00Left-click to toggle AFK status.")
	end,
})


function addon:ADDON_LOADED(addon)
	if addon == "Broker_AFKTimer" then
		Broker_AFKTimerData = Broker_AFKTimerData or {
			showDuration = true,
			localTime = false,
		}
		db = Broker_AFKTimerData
		-- use saved variables to remember AFK session after a UI reload
		if db.note then
			self:SetScript("OnUpdate", onUpdate)
		end
		self:UnregisterEvent("ADDON_LOADED")
	end
end


function addon:PLAYER_ENTERING_WORLD()
	-- when "entering world" (not on login) AFK is always cancelled, but no system message is sent, so we need a custom made check
	if UnitIsAFK("player") and db.note then
		self:EndAFK()
		DEFAULT_CHAT_FRAME:AddMessage(CLEARED_AFK, 1, 1, 0)
	end
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end


function addon:PLAYER_LOGIN()
	self:UnregisterEvent("PLAYER_GUILD_UPDATE")
end


function addon:PLAYER_LEAVING_WORLD()
	if db.note then
		self:RegisterEvent("PLAYER_ENTERING_WORLD")
	end
end


function addon:PLAYER_GUILD_UPDATE()
	self:SetScript("OnUpdate", nil)
	db.note = nil
	db.startUptime = nil
	db.startTimestamp = nil
	self:UnregisterEvent("PLAYER_LOGIN")
	self:UnregisterEvent("PLAYER_GUILD_UPDATE")
end


function addon:CHAT_MSG_SYSTEM(msg)
	-- try to extract an AFK note from the chat system message..
	local note = msg:match(MARKED_AFK_MESSAGE:gsub("$1", ""):gsub("%%s", "(.+)"))
	
	-- ..and if it worked, it was an indication of the player going AFK
	if note then
		local startHr, startMin
		
		-- local or server time?
		if db.localTime then
			local time = date("*t")
			startHr = time.hour
			startMin = time.min
		else
			startHr, startMin = GetGameTime()
		end
		
		db.note = note
		-- get time stamp when AFK starts
		db.startUptime = GetTime() -- get system uptime when AFK starts
		db.startTimestamp = format("%d:%.2d", startHr, startMin)
		duration = 0
		
		self.dataobj.text = "AFK since "..db.startTimestamp..(db.showDuration and " ("..addon:SecondsToTime(duration)..")" or "")
		
		-- start counting timer
		self:SetScript("OnUpdate", onUpdate)
	elseif msg == CLEARED_AFK then
		self:EndAFK()
	end
end


function addon:EndAFK()
	-- stop counting timer
	self:SetScript("OnUpdate", nil)
	
	self.dataobj.text = "Not AFK"
	
	-- print AFK time
	DEFAULT_CHAT_FRAME:AddMessage("You were AFK for "..self:SecondsToTime(duration, true).." (since "..db.startTimestamp..")", 1, 1, 0)
	
	db.note = nil
	db.startUptime = nil
	db.startTimestamp = nil
end


-- converts seconds into minutes and seconds
function addon:SecondsToTime(seconds, verbose)
	local hours, minutes = 0, 0
	if seconds >= 3600 then
		hours = floor(seconds / 3600)
		minutes = floor((seconds % 3600) / 60)
		seconds = seconds % 60
	elseif seconds >= 60 then
		hours = 0
		minutes = floor(seconds / 60)
		seconds = seconds % 60
	end
	
	if verbose then
		return (hours > 0 and hours.." |4hour:hours;, " or "")..(minutes > 0 and minutes.." |4minute:minutes; and " or "")..seconds.." |4second:seconds;."
	else
		return format((hours > 0 and hours..":%.2" or "%").."d:%.2d", minutes, seconds)
	end
end